==> dvcx-0.55.0-py3-none-any.whl: dql/sql/types.py <==
"""
SQL types.

This module provides SQL types to provide common features and interoperability
between different database backends which often have different typing systems.

See https://docs.sqlalchemy.org/en/20/core/custom_types.html#sqlalchemy.types.TypeDecorator.load_dialect_impl

For the corresponding python to db type conversion, it's often simpler and
more direct to use methods at the DBAPI rather than sqlalchemy. For example
for sqlite we can use `sqlite.register_converter`
( https://docs.python.org/3/library/sqlite3.html#sqlite3.register_converter )
"""

from types import MappingProxyType
from typing import Any, Dict

from sqlalchemy import TypeDecorator, types

_registry: Dict[str, "TypeConverter"] = {}
registry = MappingProxyType(_registry)
types_defaults = None


def register_backend_types(dialect_name: str, type_cls):
    _registry[dialect_name] = type_cls


def register_backend_types_defaults(td: "TypeDefaults"):
    global types_defaults  # pylint: disable=W0603  # noqa: PLW0603
    types_defaults = td


def converter(dialect) -> "TypeConverter":
    name = dialect.name
    try:
        return registry[name]
    except KeyError:
        raise ValueError(  # noqa: B904
            f"No type converter registered for dialect: {dialect.name!r}"
        )


# pylint: disable=abstract-method


class SQLType(TypeDecorator):
    impl = types.TypeEngine
    cache_ok = True

    def serialize(self) -> Dict[str, Any]:
        return {"type": self.__class__.__name__}


class String(SQLType):
    impl = types.String

    def load_dialect_impl(self, dialect):
        return converter(dialect).string()

    @staticmethod
    def default_value():
        return types_defaults.string()


class Boolean(SQLType):
    impl = types.Boolean

    def load_dialect_impl(self, dialect):
        return converter(dialect).boolean()

    @staticmethod
    def default_value():
        return types_defaults.boolean()


class Int(SQLType):
    impl = types.INTEGER

    def load_dialect_impl(self, dialect):
        return converter(dialect).int()

    @staticmethod
    def default_value():
        return types_defaults.int()


class Int32(Int):
    def load_dialect_impl(self, dialect):
        return converter(dialect).int32()

    @staticmethod
    def default_value():
        return types_defaults.int32()


class Int64(Int):
    def load_dialect_impl(self, dialect):
        return converter(dialect).int64()

    @staticmethod
    def default_value():
        return types_defaults.int64()


class Float(SQLType):
    impl = types.INTEGER

    def load_dialect_impl(self, dialect):
        return converter(dialect).float()

    @staticmethod
    def default_value():
        return types_defaults.float()


class Float32(Int):
    def load_dialect_impl(self, dialect):
        return converter(dialect).float32()

    @staticmethod
    def default_value():
        return types_defaults.float32()


class Float64(Int):
    def load_dialect_impl(self, dialect):
        return converter(dialect).float64()

    @staticmethod
    def default_value():
        return types_defaults.float64()


class Array(SQLType):
    impl = types.ARRAY

    def load_dialect_impl(self, dialect):
        return converter(dialect).array(self.item_type)

    def serialize(self) -> Dict[str, Any]:
        return {
            "type": self.__class__.__name__,
            "item_type": self.item_type.serialize(),
        }

    @staticmethod
    def default_value():
        return types_defaults.array()


class JSON(SQLType):
    impl = types.JSON

    def load_dialect_impl(self, dialect):
        return converter(dialect).json()

    @staticmethod
    def default_value():
        return types_defaults.json()


class DateTime(SQLType):
    impl = types.DATETIME

    def load_dialect_impl(self, dialect):
        return converter(dialect).datetime()

    @staticmethod
    def default_value():
        return types_defaults.datetime()


class Binary(SQLType):
    impl = types.BINARY

    def load_dialect_impl(self, dialect):
        return converter(dialect).binary()

    @staticmethod
    def default_value():
        return types_defaults.binary()


# pylint: enable=abstract-method


class TypeConverter:
    def string(self):
        return types.String()

    def boolean(self):
        return types.Boolean()

    def int(self):
        return types.Integer()

    def int32(self):
        return self.int()

    def int64(self):
        return self.int()

    def float(self):
        return types.Float()

    def float32(self):
        return self.float()

    def float64(self):
        return self.float()

    def array(self, item_type):
        return types.ARRAY(item_type)

    def json(self):
        return types.JSON()

    def datetime(self):
        return types.DATETIME()

    def binary(self):
        return types.BINARY()


class TypeDefaults:
    def string(self):
        return None

    def boolean(self):
        return None

    def int(self):
        return None

    def int32(self):
        return None

    def int64(self):
        return None

    def float(self):
        return None

    def float32(self):
        return None

    def float64(self):
        return None

    def array(self):
        return None

    def json(self):
        return None

    def datetime(self):
        return None

    def uuid(self):
        return None

    def binary(self):
        return None


register_backend_types("default", TypeConverter())
register_backend_types_defaults(TypeDefaults())

==> dvcx-0.58.0-py3-none-any.whl: dql/sql/types.py <==
"""
SQL types.

This module provides SQL types to provide common features and interoperability
between different database backends which often have different typing systems.

See https://docs.sqlalchemy.org/en/20/core/custom_types.html#sqlalchemy.types.TypeDecorator.load_dialect_impl

For the corresponding python to db type conversion, it's often simpler and
more direct to use methods at the DBAPI rather than sqlalchemy. For example
for sqlite we can use `sqlite.register_converter`
( https://docs.python.org/3/library/sqlite3.html#sqlite3.register_converter )
"""

from types import MappingProxyType
from typing import Any, Dict, Union

from sqlalchemy import TypeDecorator, types
from sqlalchemy.sql.visitors import TraversibleType

_registry: Dict[str, "TypeConverter"] = {}
registry = MappingProxyType(_registry)

_read_converter_registry: Dict[str, "TypeReadConverter"] = {}
read_converter_registry = MappingProxyType(_read_converter_registry)

types_defaults = None


def register_backend_types(dialect_name: str, type_cls):
    _registry[dialect_name] = type_cls


def register_backend_types_defaults(td: "TypeDefaults"):
    global types_defaults  # noqa: PLW0603
    types_defaults = td


def register_type_read_converters(dialect_name: str, trc: "TypeReadConverter"):
    _read_converter_registry[dialect_name] = trc


def converter(dialect) -> "TypeConverter":
    name = dialect.name
    try:
        return registry[name]
    except KeyError:
        raise ValueError(  # noqa: B904
            f"No type converter registered for dialect: {dialect.name!r}"
        )


def read_converter(dialect) -> "TypeReadConverter":
    name = dialect.name
    try:
        return read_converter_registry[name]
    except KeyError:
        raise ValueError(  # noqa: B904
            f"No read type converter registered for dialect: {dialect.name!r}"
        )


class SQLType(TypeDecorator):
    impl = types.TypeEngine
    cache_ok = True

    def to_dict(self) -> Dict[str, Any]:
        return {"type": self.__class__.__name__}

    @classmethod
    def from_dict(cls, _: Dict[str, Any]) -> Union[TraversibleType, "SQLType"]:
        return cls


class String(SQLType):
    impl = types.String

    def load_dialect_impl(self, dialect):
        return converter(dialect).string()

    @staticmethod
    def default_value():
        return types_defaults.string()

    @staticmethod
    def on_read_convert(value, dialect):
        return read_converter(dialect).string(value)


class Boolean(SQLType):
    impl = types.Boolean

    def load_dialect_impl(self, dialect):
        return converter(dialect).boolean()

    @staticmethod
    def default_value():
        return types_defaults.boolean()

    @staticmethod
    def on_read_convert(value, dialect):
        return read_converter(dialect).boolean(value)


class Int(SQLType):
    impl = types.INTEGER

    def load_dialect_impl(self, dialect):
        return converter(dialect).int()

    @staticmethod
    def default_value():
        return types_defaults.int()

    @staticmethod
    def on_read_convert(value, dialect):
        return read_converter(dialect).int(value)


class Int32(Int):
    def load_dialect_impl(self, dialect):
        return converter(dialect).int32()

    @staticmethod
    def default_value():
        return types_defaults.int32()

    @staticmethod
    def on_read_convert(value, dialect):
        return read_converter(dialect).int32(value)


class Int64(Int):
    def load_dialect_impl(self, dialect):
        return converter(dialect).int64()

    @staticmethod
    def default_value():
        return types_defaults.int64()

    @staticmethod
    def on_read_convert(value, dialect):
        return read_converter(dialect).int64(value)


class Float(SQLType):
    impl = types.INTEGER

    def load_dialect_impl(self, dialect):
        return converter(dialect).float()

    @staticmethod
    def default_value():
        return types_defaults.float()

    @staticmethod
    def on_read_convert(value, dialect):
        return read_converter(dialect).float(value)


class Float32(Int):
    def load_dialect_impl(self, dialect):
        return converter(dialect).float32()

    @staticmethod
    def default_value():
        return types_defaults.float32()

    @staticmethod
    def on_read_convert(value, dialect):
        return read_converter(dialect).float32(value)


class Float64(Int):
    def load_dialect_impl(self, dialect):
        return converter(dialect).float64()

    @staticmethod
    def default_value():
        return types_defaults.float64()

    @staticmethod
    def on_read_convert(value, dialect):
        return read_converter(dialect).float64(value)


class Array(SQLType):
    impl = types.ARRAY

    def load_dialect_impl(self, dialect):
        return converter(dialect).array(self.item_type)

    def to_dict(self) -> Dict[str, Any]:
        item_type_dict = (
            self.item_type.to_dict()
            if isinstance(self.item_type, SQLType)
            else self.item_type().to_dict()
        )
        return {
            "type": self.__class__.__name__,
            "item_type": item_type_dict,
        }

    @classmethod
    def from_dict(cls, d: Dict[str, Any]) -> Union[TraversibleType, SQLType]:
        sub_t = NAME_TYPES_MAPPING[d["item_type"]["type"]].from_dict(  # type: ignore [attr-defined]
            d["item_type"]
        )
        return cls(sub_t)

    @staticmethod
    def default_value():
        return types_defaults.array()

    @staticmethod
    def on_read_convert(value, dialect):
        return read_converter(dialect).array(value)


class JSON(SQLType):
    impl = types.JSON

    def load_dialect_impl(self, dialect):
        return converter(dialect).json()

    @staticmethod
    def default_value():
        return types_defaults.json()

    @staticmethod
    def on_read_convert(value, dialect):
        return read_converter(dialect).json(value)


class DateTime(SQLType):
    impl = types.DATETIME

    def load_dialect_impl(self, dialect):
        return converter(dialect).datetime()

    @staticmethod
    def default_value():
        return types_defaults.datetime()

    @staticmethod
    def on_read_convert(value, dialect):
        return read_converter(dialect).datetime(value)


class Binary(SQLType):
    impl = types.BINARY

    def load_dialect_impl(self, dialect):
        return converter(dialect).binary()

    @staticmethod
    def default_value():
        return types_defaults.binary()

    @staticmethod
    def on_read_convert(value, dialect):
        return read_converter(dialect).binary(value)


class TypeReadConverter:
    def string(self, value):
        return value

    def boolean(self, value):
        return value

    def int(self, value):
        return value

    def int32(self, value):
        return value

    def int64(self, value):
        return value

    def float(self, value):
        return value

    def float32(self, value):
        return value

    def float64(self, value):
        return value

    def array(self, value):
        return value

    def json(self, value):
        return value

    def datetime(self, value):
        return value

    def uuid(self, value):
        return value

    def binary(self, value):
        return value


class TypeConverter:
    def string(self):
        return types.String()

    def boolean(self):
        return types.Boolean()

    def int(self):
        return types.Integer()

    def int32(self):
        return self.int()

    def int64(self):
        return self.int()

    def float(self):
        return types.Float()

    def float32(self):
        return self.float()

    def float64(self):
        return self.float()

    def array(self, item_type):
        return types.ARRAY(item_type)

    def json(self):
        return types.JSON()

    def datetime(self):
        return types.DATETIME()

    def binary(self):
        return types.BINARY()


class TypeDefaults:
    def string(self):
        return None

    def boolean(self):
        return None

    def int(self):
        return None

    def int32(self):
        return None

    def int64(self):
        return None

    def float(self):
        return None

    def float32(self):
        return None

    def float64(self):
        return None

    def array(self):
        return None

    def json(self):
        return None

    def datetime(self):
        return None

    def uuid(self):
        return None

    def binary(self):
        return None


register_backend_types("default", TypeConverter())
register_type_read_converters("default", TypeReadConverter())
register_backend_types_defaults(TypeDefaults())

TYPES = [
    String,
    Boolean,
    Int,
    Int32,
    Int64,
    Float,
    Float32,
    Float64,
    Array,
    JSON,
    DateTime,
    Binary,
]

NAME_TYPES_MAPPING = {t.__name__: t for t in TYPES}
